<?php

class Braintree_Gateway_Merchant_Account_Settings extends Braintree_Gateway_Settings_API
{

	public function __construct()
	{
		$this->page = 'braintree-gateway-settings';
		$this->id = 'checkout_settings';
		$this->tab = 'checkout-settings';
		$this->title = array (
				'title' => __( 'Merchant Accounts', 'braintree-payments' ), 
				'description' => __( 'Your merchant accounts determine the currency that your transactions are settled in. If you are using a currency switcher, then the merchant account will be chosen based on the order\'s currency. By default, the plugin loads all of your merchant accounts when you save your API keys.', 'braintree-payments' ) 
		);
		add_filter( 'braintree_gateway_default_settings', array (
				$this, 
				'get_default_settings' 
		) );
		add_action( "braintree_gateway_{$this->tab}_save_settings", array (
				$this, 
				'save' 
		) );
		add_filter( 'braintree_settings_localized_variables', array (
				$this, 
				'localize_vars' 
		) );
		add_filter( 'braintree_gateway_validate_woocommerce_braintree_sandbox_merchant_account_id', array (
				$this, 
				'validate_merchant_accounts' 
		), 10, 2 );
	}

	public function localize_vars( $vars )
	{
		$vars [ 'merchant_accounts' ] = array (
				'sandbox' => array (
						'text' => __( 'Merchant Account (%s)', 'braintree-payments' ), 
						'html' => '<div class="col s8"><label>' . __( 'Sandbox Merchant Account (%currency%)', 'braintree-payments' ) . '</label><input type="text" name="%name%" value="" placeholder="' . __( 'Enter Merchant Account', 'braintree-payments' ) . '"/><i class="material-icons trash-merchant-account">delete</i></div>' 
				), 
				'messages' => array (
						'no_currency' => __( 'Please select a currency from the dropdown so you can add your merchant account.', 'braintree-payments' ), 
						'currency_exists' => __( 'You already have a merchant account configured for currency %currency%', 'braintree-payments' ) 
				) 
		);
		return $vars;
	}

	public function settings()
	{
		return array (
				'woocommerce_braintree_sandbox_merchant_account_id' => array (
						'type' => 'custom', 
						'title' => __( 'Sandbox Merchant Accounts', 'braintree-payments' ), 
						'save' => true, 
						'default' => array (), 
						'function' => array (
								$this, 
								'output_merchant_account_settings' 
						), 
						'attributes' => array (
								'environment' => 'sandbox', 
								'data-setting' => $this->get_field_key_name( 'woocommerce_braintree_sandbox_merchant_account_id' ) . '[%currency%]' 
						), 
						'label' => __( 'Sandbox Merchant Account (%s)', 'braintree-payments' ), 
						'tool_tip' => true, 
						'description' => __( 'Your merchant accounts determine the settlement currency of your transactions.', 'braintree-payments' ) 
				) 
		);
	}

	public function output_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'html-helpers/settings-modal.php';
	}

	public function output_merchant_account_settings( $key, $data )
	{
		$field_key = $this->get_field_key_name( $key );
		$data = wp_parse_args( $data, $this->get_default_custom_html_args() );
		include bt_manager()->plugin_admin_path() . 'views/merchant-accounts.php';
	}

	public function validate_merchant_accounts( $value, $key )
	{
		if ( empty( $value ) ) { // array of merchant accounts is empty so exit.
			return $value;
		}
		$environment = 'sandbox';
		
		// compare values, maybe an API call isn't needed if the vales haven't changed.
		$old_value = bt_manager()->get_option( $key );
		
		if ( $old_value === $value ) {
			return $value;
		}
		
		foreach ( $value as $currency => $account ) {
			if ( empty( $account ) ) {
				bt_manager()->add_admin_notice( 'error', __( 'Values for merchant accounts cannot be blank', 'braintree-payments' ) );
				return false;
			}
			Braintree_Configuration::environment( "sandbox" );
			Braintree_Configuration::merchantId( bt_manager()->get_option( "sandbox_merchant_id" ) );
			Braintree_Configuration::privateKey( bt_manager()->get_option( "sandbox_private_key" ) );
			Braintree_Configuration::publicKey( bt_manager()->get_option( "sandbox_public_key" ) );
			try {
				$merchant_account = Braintree_MerchantAccount::find( $account );
			} catch( Braintree\Exception\NotFound $e ) {
				bt_manager()->add_admin_notice( 'error', sprintf( __( 'Merchant account %s could not be found in your sandbox environment.', 'braintree-payments' ), $account ) );
				return false;
			} catch( \Braintree\Exception $e ) {
				bt_manager()->add_admin_notice( 'error', sprintf( __( 'There was an error retrieving merchant account %s. Reason: %s', 'braintree-payments' ), $account, $e->getMessage() ) );
				return false;
			}
		}
		return $value;
	}
}